//
// Created by fluffymoo on 11.6.18.
//

#ifndef INC_4NEURO_NEURONLOGISTIC_H
#define INC_4NEURO_NEURONLOGISTIC_H

#include <cmath>
#include "neuron.h"
#include "../constants.h"

class NeuronLogistic:Neuron {
    /**
     * Constructs the object of the Logistic neuron with activation function
     * f(x) = (1 + e^(-x + b))^(-a)
     * @param a First coefficient, stored in activation_function_parameters[0]
     * @param b Second coefficient, stored in activation_function_parameters[1]
     */
    explicit NeuronLogistic(double a = 0.0, double b = 0.0);

    /**
     * Evaluates '(1 + e^(-x + b))^(-a)' and stores the result into the 'state' property
     */
    void activate() override;

    /**
     * Calculates the partial derivative of the activation function
     * f(x) = (1 + e^(-x + b))^(-a)
     * @param param_idx Index of the parameter to calculate derivative of
     * @return Partial derivative of the activation function according to the
     * 'param_idx'-th parameter.
     */
    double activation_function_get_partial_derivative(int param_idx) override;

    /**
     * Calculates d/dx of (1 + e^(-x + b))^(-a)
     * @return a * e^(b - x) * [e^(b - x) + 1]^(-a)
     */
    double activation_function_get_derivative() override;
};


#endif //INC_4NEURO_NEURONLOGISTIC_H
/*
!-------------------------!--------------------------------------------------------------------
    ! class logistic_neuron_t !
    !-------------------------!

    !> Logistic neuron class - uses generalised logistic function as an activation function
    !! in the form f(x) = (1 + e^(-x))^(-alpha),
    !! 'x' being the neuron potential here
    type, extends(neuron_t) :: logistic_neuron_t
        private

        real(kind=real_4neuro) :: alpha_coef  !< The alpha coefficient used in the activation function

        contains

            !> Activation function - generalised logistic f.
            procedure, private :: activate => logistic_activate_impl
    end type logistic_neuron_t

    interface logistic_neuron_t
        !> Non-parametric constructor of logistic_neuron_t class
        !! Alpha coefficient is set to 1
        !! @return An instance of the class logistic_neuron_t
        module procedure :: new_logistic_neuron

        !> Constructor of the logistic_neuron_t class
        !! @param[in] alpha_coef Alpha coefficient in the logistic activation function
        !! @return An instance of the class logistic_neuron_t
        module procedure :: new_logistic_neuron_1
    end interface logistic_neuron_t
 */